// log on screen
#define LG_ON

// include
#include "library/interface.h"

// spring specific
#include "bayes/feature_table.h"
#include "bayes/feature_bayes.h"
#include <stdio.h>
#include <time.h>
#include <stdlib.h>

// bayes for interaction prediction
struct ModSpring_Bayes
{
    // storage
    Storage store;

    // output file
    fstream ftrain;

    // construct
    void construct (string path)
    {
        // number of features
        int nfeat  = 5;

        // loading features
        Msg::write("Loading features...");

        // load features for training
        Feature_Table train;
        train.construct(nfeat);
        
        // add spring threading score to classifier
        train.addfeature(path + "/feature_0.txt", 0, 0.25); 
        
        // load other features
	for (int i = 1; i < nfeat; i++) 
	train.addfeature(path + "/feature_" + Convert::toString(i) + ".txt", i);

        // loading features
        Msg::write("Loading reference...");
	

        // data size
	
        int ndata = train.ftable.size();
	cout <<ndata<<endl;
	for (int i = 0 ; i < ndata ; i ++ )
	cout <<train.fid[i]<<endl;
       

        // separate
        int nhalf = ndata;

        // loading features
        Msg::write("Initializing network training on %i out of %i interactions...", nhalf, ndata);
        

        // construct classifier
        Feature_Bayes ba; 
        ba.construct(nfeat, path);
	

        ba.train(train, nhalf);

        // plot table
        //ba.show(train, train.ftable.size());
        ba.show_probs();

        // log
        Msg::write("Network loaded.");

        // arrays
        Vec < int >    lref;
        Vec < string > lid;
        Vec < double > lcon;

        // get interface size
        for (int i = 0; i < ndata; i++)
        {
            // check if its in set
            string id = train.fid[i];
            int type = ba.href.get(id);
            if (type == -1)
                continue;

            // confidence
            double confidence = ba.predict(train.ftable[i]);

            // add to list
            lid.push_back(id);
            lcon.push_back(confidence);
            lref.push_back(type);
        }

        // make index
        Vec < int > lindex;
        lindex.resize(lid.size());
        for (int i = 0; i < lindex.size(); i++)
            lindex[i] = i;
	//for (int i =0; i < lindex.size(); i++)
	// cout <<lid[i]<<"  "<<lcon[i]<<"  "<<lref[i]<<endl;
        // sort
        Lib::sort_greater(lcon, lindex);

        // log
        Msg::write("Prediction completed for %i (feature 0-4 match GS) with %i/%i pairs.", lid.size(), ba.npos, ba.nneg);

        // make output file used for draw Roc curce with increasing the predicted pair
        // froc is correct, but the threshhold of confidence is not correct 
        fstream froc;
        froc.open ("roc.txt", ios_base::out);

        // predict
        double maxmcc = 0.0;
        double maxfp  = 0.0;
        double maxtp  = 0.0;
        double maxcon = 0.0;
        double tp = 0, fp = 0;
	//cout <<"%%%%%%%%%%%%%%%%%%%%%%%"<<endl;
	//cout <<lindex.size()<<endl;
        for (int i = 0; i < lindex.size(); i++)
        {
            // comparison

	  	 
            if (lref[lindex[i]] == Feature_Bayes::TP)
	       tp++;
            else
                fp++;

            // get mathew correlation coefficiant
            double fn = ba.npos - tp;
            double tn = ba.nneg - fp;
            double mcc = 0.0;
            double denom = (tp+fp)*(tp+fn)*(tn+fp)*(tn+fn);
            if (denom > 0.0)
            {
                mcc = (tp*tn-fp*fn)/sqrt(denom);
		//cout <<lid[lindex[i]]<<"  "<<lcon[i]<<"  "<<lref[lindex[i]]<<"  "<<mcc<<endl;
                if (mcc >= maxmcc)
                {
                    maxmcc = mcc;
                    maxcon = lcon[i];
                    maxfp  = fp;
                    maxtp  = tp;
                }
            }

            // print
            //if (i % (lindex.size() / 10) == 0 || i == lindex.size() - 1)
	    froc << fp / ba.nneg << " " << tp / ba.npos << " " << mcc << endl;
		//cout << lref[lindex[i]] << " "  <<fp <<" " << ba.nneg << " "<< tp <<" "<< ba.npos << endl;
		//cout <<tp<<","<<fp<<endl;

        }

        // final
        froc << maxfp<<" "<<maxtp<<" "<<maxfp / ba.nneg << " " << maxtp / ba.npos << " " << maxmcc << endl;

        // close
        froc.close();

        // count number of predictions 
        // interactome.txt is not correct because of the uncorrect threshhold of confidence
        fstream fout;
        fout.open("interactome.txt", ios_base::out);
        fout << "Please contact aysam.guerler@gmail.com or zhng@umich.edu for questions." << endl;
        fout << "---------------------------------------------------------------------------------" << endl;

        // write header
        fout << "PROTEINS\tBAYES\tZ[AB-MAX]";
        for (int i = 1; i < nfeat; i++)
            fout << "\t" + train.fname[i];
        fout << endl;
        fout << "---------------------------------------------------------------------------------" << endl;

        // write predictions
        int npredicted = 0;
        for (int i = 0; i < ndata; i++)
        {
            // confidence
            double confidence = ba.predict(train.ftable[i]);

            // count
            if (confidence >= maxcon)
            {

                fout << train.fid[i] << "\t" << confidence << "\t" << int(train.fscore[i][0] * 100.0) << "\t";
                for (int j = 1; j < nfeat; j++)
                    fout << "\t" << train.fscore[i][j];
                fout << endl;
                npredicted++;
            }
        }
        fout << "---------------------------------------------------------------------------------" << endl;
        fout.close();
        
        // log to screen
        cout << endl;
	cout << "not correct because not consider all the pair with confidence" << maxcon <<" , but roc can be used to draw roc curve "<< endl;
        cout << "Threshold                     : " << maxcon << endl;
        cout << "Total predictions in "<< ndata<<"     :"<< npredicted << endl;
        cout << "MCC on test set               : " << maxmcc << endl;

	cout <<"----------------------------------------------------------------------------------"<< endl;

	///////////////////////////////////////////////////////////////////////////////////////////////////////////////
	// obtain optimized confidence that let the mcc biggest 
	
	// frocfig.txt not good for drawing roc curve because tp and fp changes with confidence
	// confidence is not continue ! thus likely for different confidences such as 1.2  1.3  1.4, the tp and fp are not changed. 
	fstream frocfig;
	frocfig.open ("frocfig.txt", ios_base::out);

		maxmcc = 0.0;
        maxfp  = 0.0;
        maxtp  = 0.0;
        maxcon = 0.0;
	double conthr;
	
       // obtain the optimized confidence
       // because the maxmum confidence is 43
	for (int l = 0 ; l < 5000; l ++ )
	  {
	    conthr = 1.0 + 0.01 * (double) l;
	    // there are different tp and fp for different conthr
	    tp = 0, fp = 0;
	     // obtain tp and fp 
	      for (int i = 0; i < lindex.size(); i++)
        {
            // comparison
			if ( lcon[i] >= conthr ) 
			{
				if (lref[lindex[i]] == Feature_Bayes::TP) 
					tp++;
				else  fp++;
				}
		}

            // get mathew correlation coefficiant
            double fn = ba.npos - tp;
            double tn = ba.nneg - fp;
            double mcc = 0.0;
            double denom = (tp+fp)*(tp+fn)*(tn+fp)*(tn+fn);
            if (denom > 0.0)
            {
                mcc = (tp*tn-fp*fn)/sqrt(denom);
               
            }
	// obtain maxmcc for all conthr
	    if (mcc >= maxmcc)
                {
                    maxmcc = mcc;
                    maxcon = conthr;
                    maxfp  = fp;
                    maxtp  = tp;
                }
            // print fp and tp for one conthr
	    frocfig <<fp / ba.nneg << " " << tp / ba.npos << " " << mcc << endl;
	  }
	// at the end of frocfig.txt, print maxcc for optimized confidence
	frocfig  <<maxfp / ba.nneg << " " << maxtp / ba.npos << " " << maxmcc <<" " << maxcon <<endl;


        // close
        frocfig.close();

        // count number of predictions in 48462
        // interactome_true.txt is useful because the correct threshhold of confidence
        
        fout.open("interactome_true.txt", ios_base::out);
        fout << "Please contact aysam.guerler@gmail.com or zhng@umich.edu for questions." << endl;
        fout << "---------------------------------------------------------------------------------" << endl;

        // write header
        fout << "PROTEINS\tBAYES\tZ[AB-MAX]";
        for (int i = 1; i < nfeat; i++)
            fout << "\t" + train.fname[i];
        fout << endl;
        fout << "---------------------------------------------------------------------------------" << endl;

        // write predictions
        npredicted = 0;
        for (int i = 0; i < ndata; i++)
        {
            // confidence
            double confidence = ba.predict(train.ftable[i]);
            // count
            if (confidence >= maxcon)
            {

	      fout << train.fid[i].substr(0, 5)<<"\t"<<train.fid[i].substr(6, 5) << "\t" << confidence << "\t" << int(train.fscore[i][0] * 100.0) << "\t";
                for (int j = 1; j < nfeat; j++)
                    fout << "\t" << train.fscore[i][j];
                fout << endl;
                npredicted++;
            }
        }
        fout << "---------------------------------------------------------------------------------" << endl;
        fout.close();
        
        // log to screen
	cout <<" correct , after obtaining  optimized threshhold of confidence "<<endl;
        cout << endl;
        cout << "Threshold                     : " << maxcon << endl;
        cout << "Total predictions in "<<ndata<<"     : " << npredicted << endl;
        cout << "MCC on test set               : " << maxmcc << endl;

	cout <<"----------------------------------------------------------------------------------"<< endl;
    

	
	//////////////////////////////////////////////////////////////////////////////////////////////////////


	cout <<endl;
	cout <<endl;
	cout <<"***********************************************************************************"<<endl;
	cout <<"5-fold to get the last MCC of 4f and 5f methods"<<endl;

	// read GS positive 
	Feature_Table gs_pos, gs_neg;
	gs_pos.construct(1);
	gs_pos.addgs(path + "/positive.txt",0,true);
	// cout positive num
	int gs_pos_sum = gs_pos.fid.size();
	Msg::write("GS positive %i entries loaded for GS all positive .", gs_pos_sum, "GS_positive");
	

	// read GS negative
	gs_neg.construct(1);
        gs_neg.addgs(path + "/negative.txt",0,false);
	// count negative num
	int gs_neg_sum = gs_neg.fid.size();
	Msg::write("GS negative %i entries loaded for GS all negative .", gs_neg_sum, "GS_negative");


	// random sort for 0-gs_pos_sum and 0-gs_neg_sum numbers
        Vec <int> num_pos,num_neg;
        num_pos.resize(gs_pos_sum);
        num_neg.resize(gs_neg_sum);
	
         for (int i = 0; i < gs_pos_sum; i ++)
          {
             num_pos[i] = i;
          }
	// perform 5 times random for 0-gs_pos_sum
         for ( int i = 0 ; i < 5; i ++)
           {
              srand(time(0));
              for (int j = gs_pos_sum-1; j > 0; j--)
                {
                   int idex = rand()%j;
                   if (idex == j) continue;
                   int tmp = num_pos [idex];
                   num_pos [idex] = num_pos [j];
                   num_pos [j] = tmp;
                 }
           }
          
	 // random for gs_neg_sum
          for (int i = 0; i < gs_neg_sum; i ++)
          {
             num_neg [i] = i;
          }
         for ( int i = 0 ; i < 5; i ++)
           {
              srand(time(0));
              for (int j = gs_neg_sum-1; j > 0; j--)
                {
                   int idex = rand()%j;
                   if (idex == j) continue;
                   int tmp = num_neg [idex];
                   num_neg [idex] = num_neg [j];
                   num_neg [j] = tmp;
                 }
           }

	
	// 5 fold 
         int fold = 5;
	//number of pairs in each part, total part is 5 
         int nppart;
	//gs_pos_set_num, file name of 5 parts of positive samples 
	 fstream fpos;
	 string gs_pos_set;
	// 201/5 =40.2 thus 41 pairs in former 4 files, 37 in the 5th file 
	 nppart = int (gs_pos_sum/fold)+1;
	 //create gs_pos_set and gs_neg_set files
	

          for (int i = 0 ; i < fold ; i++)
            {
	      // create 5 gs_pos_set files
	      gs_pos_set = "gs_pos_set_" + Convert::toString(i) + ".txt";
              fpos.open (gs_pos_set.c_str(), ios_base::out);
              //fpos <<"part "<< i << endl;
              if ( i == fold - 1 )
                {
                 for (int j = nppart * i; j < gs_pos_sum ; j++)
		   fpos <<gs_pos.fid[num_pos[j]]<<" "<<gs_pos.ftable[num_pos[j]][0] <<endl;
                } else {
                    for (int j = nppart * i; j < nppart * (i+1) ; j++)
		      fpos << gs_pos.fid[num_pos[j]]<<" "<< gs_pos.ftable[num_pos[j]][0] <<endl;
                }

              fpos.close();
            }
	  	
	 

	  // create 5 gs_neg_set
	   fstream fneg;
	   // for (int i = 0 ; i < gs_neg_sum ; i++ )
	   //cout <<gs_neg.fid[num_neg[i]]<<" "<<gs_neg.ftable[num_neg[i]][0] <<endl;
	   nppart  = int (gs_neg_sum/fold)+1;
           for (int i = 0 ; i < fold ; i++)
            {
              string gs_neg_set = "gs_neg_set_"+ Convert::toString(i) + ".txt";
              fneg.open (gs_neg_set.c_str(), ios_base::out);

              //fneg <<"part"<< i << endl;
              if (i == fold - 1 )
                {
                 for (int j = nppart * i; j < gs_neg_sum ; j++)
                 fneg <<gs_neg.fid[num_neg[j]]<<" "<<gs_neg.ftable[num_neg[j]][0] <<endl;
                } else {
                    for (int j = nppart * i; j < nppart * (i+1) ; j++)
                    fneg <<gs_neg.fid [num_neg[j]]<<" "<<gs_neg.ftable[num_neg[j]][0] <<endl;
                }
              fneg.close();
            }

	   // combine 4 parts of gs_pos_set and gs_neg_net as training set, do fold times 
	  Feature_Table comfile;
	  fstream fgs_train_set;
	  
	   for (int i = 0; i < fold; i++)
	     {
	       comfile.construct(fold);
	       for (int j = 0; j < fold ; j++)
		 {
		   if (j == i) continue;
 
		   else {
		     comfile.addgs(path + "/gs_pos_set_" + Convert::toString(j) + ".txt", 0, true);
		     comfile.addgs(path + "/gs_neg_set_" + Convert::toString(j) + ".txt", 0, false);
		     
		   }
		 }

	       int ntrain = comfile.fid.size();
	       string gs_train_set = "gs_train_set_" + Convert::toString(i) + ".txt";
	       fgs_train_set.open (gs_train_set.c_str(), ios_base::out);
	        for (int j = 0 ; j < ntrain ; j ++ )
	           {
	            
	               fgs_train_set << comfile.fid[j] <<" "<< comfile.ftable[j][0] <<endl; 
	          }
	       fgs_train_set.close();
 
	     }




	   // strart to train  
	    Feature_Bayes bay;
	    // roc file for each train and test
	    string gs_roc;
	    
	    // combine num of tp and fp for 5 times of test 
	    Vec < double > fp_test, tp_test;
	    fp_test.resize(fold);
	    tp_test.resize(fold);


            for (int i = 0 ; i < fold ; i ++ )
            {

	      // read train to obain probs_t_train_bay and probs_n_train_bay, ie true positive and false positive for i 
               bay.construct_train_bay(fold, path, i);
	      //string id1 = "b1860 b1861";
	       //string id2 = "b3416 b3596";
	      
	       cout << endl;
	       cout <<"The " <<i<<"th training "<<endl;
               cout << "In gs_train_set_"<<i<<"  "<< "positive num: " << bay.npos_train_bay <<" "<< "  and negative num : " << bay.nneg_train_bay<<endl;
	       //cout << bay.href_train.get(id1)<<" "<<bay.href_train.get(id2) <<endl;
	       
               bay.construct_test_bay(nfeat,path,i);
	       //string id3 = "b3295 b3995";
	       //string id4 = "b1018 b3860";
	       //string id5 = "b4014 b4149";
	       //string id6 = "b0170 b0241";
	       	      
               cout <<"In test set: set gs_pos or neg_set_"<<i<<" "<< "  positive num: " << bay.npos_test_bay <<"  "<< " negative num: " << bay.nneg_test_bay<<endl;
	       
	       //cout <<bay.href_test.get(id3)<<" "<<bay.href_test.get (id4)<<" "<<bay.href_test.get (id5)<<" "<<bay.href_test.get (id6)<<endl;
		 
	       // count true positive and false positive number of 48463 for train i 
	       bay.train_bay(train, nhalf);
	       //cout <<"48463 pairs "<<"match gs_train_set_"<<i<<": "<<" true positive num: "<<" false positive num;" <<endl;
	       //cout <<bay.probs_t_train_bay[0]<<"   "<< bay.probs_n_train_bay [0]<<endl;
	       //cout <<bay.probs_t_train_bay[1]<<"   "<< bay.probs_n_train_bay [1]<<endl;
	       //cout <<bay.probs_t_train_bay[2]<<"   "<< bay.probs_n_train_bay [2]<<endl;
	       //cout <<bay.probs_t_train_bay[3]<<"   "<< bay.probs_n_train_bay [3]<<endl;
	       //cout <<bay.probs_t_train_bay[4]<<"   "<< bay.probs_n_train_bay [4]<<endl;

               //cout <<bay.npos_test_bay<<"  "<<bay.nneg_test_bay<<endl;
	       bay.show_probs_train_bay();

	       // log
	       Msg::write("Network loaded.");

	       // arrays
	       lref.clear();
	       lid.clear();
	       lcon.clear();
	       lindex.clear();

	      
	
	       for (int j = 0; j < ndata; j++)
		 {
		   // check if its in set
		   string id = train.fid[j];
		   int type = bay.href_test.get(id);
		   if (type == -1)
		     continue;

		   // confidence
		   double confidence = bay.predict_bay(train.ftable[j]);

		   // add to list
		   lid.push_back(id);
		   lcon.push_back(confidence);
		   lref.push_back(type);
		 }


	       // make index
       
	       //lid.size : the number of 48462 in test i 
	       lindex.resize(lid.size());
	       for (int j = 0; j < lindex.size(); j++)
		 lindex[j] = j;

	       // sort
	       Lib::sort_greater(lcon, lindex);

	       // log
	       Msg::write("Prediction completed for %i with %i/%i pairs.", lid.size(), bay.npos_test_bay, bay.nneg_test_bay);

	       // make output file
	       gs_roc = "gs_roc_" + Convert::toString(i) + ".txt"; 
	       froc.open (gs_roc.c_str(), ios_base::out);

	       // predict
	       double maxmcc = 0.0;
	       double maxfp  = 0.0;
	       double maxtp  = 0.0;
	       double maxcon = 0.0;
	      

	       for (int l = 0 ; l < 5000 ; l ++ )
		 {
		   conthr = 1.0 + 0.01 * (double) l ;
		   
		    double tp = 0, fp = 0;
	       for (int j = 0; j < lindex.size(); j++)
		 {
		   // comparison
		   if (lcon[j] >= conthr)
		     {
		       if (lref[lindex[j]] == Feature_Bayes::TP) tp++;
		       else fp++;
		     }
		 }



		   // get mathew correlation coefficiant
		   double fn = bay.npos_test_bay - tp;
		   double tn = bay.nneg_test_bay - fp;
		   double mcc = 0.0;
		   double denom = (tp+fp)*(tp+fn)*(tn+fp)*(tn+fn);
		   if (denom > 0.0)
		     {
		       mcc = (tp*tn-fp*fn)/sqrt(denom);
		     }

		     if (mcc >= maxmcc)
			 {
			   maxmcc = mcc;
			   maxcon = conthr;
			   maxfp  = fp;
			   maxtp  = tp;
			 }
		 

		   // print
		   //if (i % (lindex.size() / 10) == 0 || i == lindex.size() - 1)
		  froc << fp / bay.nneg_test_bay << " " << tp / bay.npos_test_bay << " " << mcc << endl;
		   //cout << lref[lindex[i]] << " "  <<fp <<" " << ba.nneg << " "<< tp <<" "<< ba.npos << endl;
		   //cout <<tp<<","<<fp<<endl;
		 
		 }

	       // final
	       //froc << maxfp<<" "<<maxtp<<" "<<maxfp / bay.nneg_test_bay << " " << maxtp / bay.npos_test_bay << " " << maxmcc << endl;
	        fp_test [i] = maxfp; 
		tp_test [i] = maxtp;

	       // close
	       froc.close();

	       // count number of predictions
	       string interactome = "interactome_" + Convert::toString(i) + ".txt"; 
       
	       fout.open(interactome.c_str(), ios_base::out);
	       fout << "Please contact aysam.guerler@gmail.com or zhng@umich.edu for questions." << endl;
	       fout << "---------------------------------------------------------------------------------" << endl;

	       // write header
	       fout << "PROTEINS\tBAYES\tZ[AB-MAX]";
	       for (int j = 1; j < nfeat; j++)
		 fout << "\t" + train.fname[j];
	       fout << endl;
	       fout << "---------------------------------------------------------------------------------" << endl;

	       // write predictions
	       int npredicted = 0;

	       
	       for (int j = 0; j < ndata; j++)
		 {
		   // confidence
		   //cout <<ndata<<endl;
		   string id = train.fid[j];
		   
		   int type = bay.href_test.get(id);
		  
		   if (type == -1)
		     continue;

		 
		   double confidence = bay.predict_bay(train.ftable[j]);
		  
		   if (confidence >= maxcon)
		     {

		       fout << train.fid[j] << "\t" << confidence << "\t" << int(train.fscore[j][0] * 100.0) << "\t";
		       for (int k = 1; k < nfeat; k++)
			 fout << "\t" << train.fscore[j][k];
		       fout << endl;
		       npredicted++;
		     }
		 }
	       fout << "---------------------------------------------------------------------------------" << endl;
	       fout.close();
        
	       // log to screen
	       cout << endl;
	       cout << "Threshold                           : " << maxcon << endl;
	       cout << "Total predictions match test set    : " << npredicted << endl;
	       cout << "MCC on test set                     : " << maxmcc << endl;

	       cout <<"----------------------------------------------------------------------------------"<< endl;
	       
	        for (int j = 0; j < ndata; j++)
		 {
		   // confidence
		   //cout <<ndata<<endl;
		   string id = train.fid[j];
		   
		 		 
		   double confidence = bay.predict_bay(train.ftable[j]);
		  
		   if (confidence >= maxcon)
		     {

		       fout << train.fid[j] << "\t" << confidence << "\t" << int(train.fscore[j][0] * 100.0) << "\t";
		       for (int k = 1; k < nfeat; k++)
			 fout << "\t" << train.fscore[j][k];
		       fout << endl;
		       npredicted++;
		     }
		 }
	       fout << "---------------------------------------------------------------------------------" << endl;
	       fout.close();
        
	       // log to screen
	       cout << endl;
	       cout << "Threshold                           : " << maxcon << endl;
	       cout << "Total predictions in 48462          : " << npredicted << endl;
	       cout << "MCC on test set                     : " << maxmcc << endl;

	       cout <<"----------------------------------------------------------------------------------"<< endl;
	      
	    }
	    
	    double tp_tot;
	    double fp_tot;
	    tp_tot = 0.0;
	    fp_tot = 0.0;

	    for (int i = 0 ; i < fold ; i ++ )
	      {
	    	
		tp_tot += tp_test [i];
		fp_tot += fp_test [i];
	
	      }
	    	double fn = ba.npos - tp_tot;
		double tn = ba.nneg - fp_tot;
		double mcc = 0.0;
		double denom = (tp_tot+fp_tot)*(tp_tot+fn)*(tn+fp_tot)*(tn+fn);
		   if (denom > 0.0)
		   mcc = (tp_tot*tn-fp_tot*fn)/sqrt(denom);
		   
		   cout <<endl;
		   cout <<"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"<<endl;
		   cout <<"the mcc of combining 5 training , TP FP rate :"<<"   "<<mcc<<"  "<<tp_tot / (tp_tot + fn )<<"  "<<fp_tot / (fp_tot + tn)<<endl;
		   cout <<"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^"<<endl;

	    
              
    }


};


// main 
int main(int n, char* arg[]) 
{ 
	// parameters
	Vec < string > input;

	// load parameters
	for (int i = 0; i < n; i++)
		input.push_back((string) arg[i]);
		
	// result 
	cout << " **************************************************************************" << endl; 
	cout << " *                             SPRING_BAYES                               *" << endl;
	cout << " *                                                                        *" << endl; 
	cout << " * Reference: A. Guerler et al.                                           *" << endl;
	cout << " * Comments on the program? Please contact: aysam.guerler@gmail.com       *" << endl;
	cout << " **************************************************************************" << endl; 
	cout << endl; 
	
	// run 
    if (n < 2)
        Msg::error ("Missing information", "[directory for feature files]");

    // check args
    ModSpring_Bayes spec;
    spec.construct(input[1]);

	// return 
	return 0; 
} 
